# Generated by Django 2.0 on 2018-05-07 09:15

from django.db import migrations, models
import django.db.models.deletion
import documents.models
import enumfields.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ElementChildren',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('ordering', models.PositiveIntegerField()),
            ],
            options={
                'db_table': 'elements_children',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ElementParents',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('ordering', models.PositiveIntegerField()),
            ],
            options={
                'db_table': 'elements_parents',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Element',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('type', enumfields.fields.EnumField(enum=documents.models.ElementType, max_length=50)),
                ('name', models.CharField(max_length=250)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ElementLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('element_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='documents.Element')),
                ('folio', models.CharField(max_length=250)),
                ('page_type', enumfields.fields.EnumField(blank=True, enum=documents.models.PageType, max_length=50, null=True)),
                ('nb', models.PositiveIntegerField(blank=True, null=True)),
                ('direction', enumfields.fields.EnumField(blank=True, enum=documents.models.PageDirection, max_length=10, null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('documents.element',),
        ),
        migrations.CreateModel(
            name='Section',
            fields=[
                ('element_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='documents.Element')),
            ],
            options={
                'abstract': False,
            },
            bases=('documents.element',),
        ),
        migrations.CreateModel(
            name='Transcription',
            fields=[
                ('element_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='documents.Element')),
                ('line', models.PositiveIntegerField(blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('score', models.FloatField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('documents.element',),
        ),
        migrations.AddField(
            model_name='elementlink',
            name='child',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='link_child', to='documents.Element'),
        ),
        migrations.AddField(
            model_name='elementlink',
            name='parent',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='link_parent', to='documents.Element'),
        ),
        migrations.AddField(
            model_name='element',
            name='parents',
            field=models.ManyToManyField(through='documents.ElementLink', to='documents.Element'),
        ),
        migrations.AlterUniqueTogether(
            name='elementlink',
            unique_together={('parent', 'order'), ('parent', 'child')},
        ),

        # Add new views
        migrations.RunSQL(
            """
            CREATE OR REPLACE RECURSIVE VIEW elements_parents(id, child_id, parent_id, ordering) AS (
                    SELECT dd.id, ddl.child_id, ddl.parent_id, ddl.order
                    FROM documents_elementlink ddl, documents_element dd
                    WHERE dd.id = ddl.child_id
                UNION
                    SELECT pd.id, ddl.child_id, ddl.parent_id, ddl.order
                    FROM documents_elementlink ddl INNER JOIN elements_parents pd
                    ON pd.parent_id = ddl.child_id
            );
            """,
            reverse_sql="DROP VIEW IF EXISTS elements_parents;"
        ),
        migrations.RunSQL(
            """
            CREATE OR REPLACE RECURSIVE VIEW elements_children(id, child_id, parent_id, ordering) AS (
                    SELECT dd.id, ddl.child_id, ddl.parent_id, ddl.order
                    FROM documents_elementlink ddl, documents_element dd
                    WHERE dd.id = ddl.parent_id
                UNION
                    SELECT cd.id, ddl.child_id, ddl.parent_id, ddl.order
                    FROM documents_elementlink ddl INNER JOIN elements_children cd
                    ON cd.child_id = ddl.parent_id
            );
            """,
            reverse_sql="DROP VIEW IF EXISTS elements_children;"
        ),
    ]
